/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * This file is part of the libe-book project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#ifndef PEANUTPRESSTYPES_H_INCLUDED
#define PEANUTPRESSTYPES_H_INCLUDED

namespace libebook
{

enum PeanutPressFontType
{
  PEANUTPRESS_FONT_TYPE_NORMAL,
  PEANUTPRESS_FONT_TYPE_STD,
  PEANUTPRESS_FONT_TYPE_BOLD,
  PEANUTPRESS_FONT_TYPE_LARGE
};

struct PeanutPressAttributes
{
  PeanutPressAttributes();

  bool pageBreak;
  bool chapter;
  bool center;
  bool right;
  bool italic;
  bool underline;
  bool overstrike;
  bool indent;
  unsigned indentPercent;
  PeanutPressFontType font;
  bool bold;
  bool superscript;
  bool subscript;
  bool smallcaps;
};

} // namespace libebook

#endif // PEANUTPRESSTYPES_H_INCLUDED

/* vim:set shiftwidth=2 softtabstop=2 expandtab: */
