/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * This file is part of the libe-book project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include "FictionBook3Parser.h"

#include <cassert>

namespace libebook
{

FictionBook3Parser::FictionBook3Parser(const RVNGInputStreamPtr_t &input, librevenge::RVNGTextInterface *document)
  : m_input(input)
  , m_document(document)
{
  assert(bool(m_input));
}

bool FictionBook3Parser::parse()
{
  return false;
}

}

/* vim:set shiftwidth=2 softtabstop=2 expandtab: */
